# Copyright 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# based on configured release version
RPC_API_VERSION = "1.0"

# API version history:
#
# 1.0 - Initial version.  (We started keeping track at icehouse-3)
# 1.1 -
# 1.2 - ...
VERSION_ALIASES = {
    'icehouse': '1.0'
}
